/*
 * Copyright (c) 2016-2017 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *                                      Anthony Mallet on Tue Aug 23 2016
 */
#ifndef H_MRSIM_CODELS
#define H_MRSIM_CODELS

#include <sys/time.h>
#include <errno.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#include "mrsim_c_types.h"

struct mrsim_pty_s {
  uint8_t buf[128], r, w; /* read ring buffer */

  bool start;
  bool escape;
  uint8_t msg[64], len; /* last message */
};

static inline genom_event
mrsim_e_sys_error(const char *s, genom_context self)
{
  mrsim_e_sys_detail d;
  size_t l = 0;

  d.code = errno;
  if (s) {
    strncpy(d.what, s, sizeof(d.what) - 3);
    l = strlen(s);
    strcpy(d.what + l, ": ");
    l += 2;
  }
  strerror_r(d.code, d.what + l, sizeof(d.what) - l);
  return mrsim_e_sys(&d, self);
}


#ifdef __cplusplus
extern "C" {
#endif
  int	mrsim_open_pty(const char *device);
  void	mrsim_close_pty(int fd, const char *device);
  int	mrsim_wait_msg(int fd);
  int	mrsim_recv_msg(int fd, struct mrsim_pty_s *pty);
  int	mrsim_send_msg(int fd, const char *fmt, ...);

  void mrsim_motor_model(const struct mrsim_ids_simparam_s_motor_s *motor,
                         struct mrsim_ids_simstate_s_motor_s *state,
                         const double dt);
  void mrsim_wrench(const struct mrsim_ids_simparam_s_body_s *body,
                    struct mrsim_ids_simstate_s *state);
  void mrsim_dynamics(const struct mrsim_ids_simparam_s_body_s *bodyp,
                      struct mrsim_ids_simstate_s_body_s *body,
                      or_pose_estimator_state *state, double dt);
  double gnoise(double stddev);
  void mrsim_addnoise(const struct mrsim_ids_simparam_s_state_s *param,
                      or_pose_estimator_state *state);

  void		mrsim_motor_pwm(struct mrsim_ids_simstate_s_motor_s *motor,
    const double pwm);
  void		mrsim_motor_period(
    struct mrsim_ids_simstate_s_motor_s *motor, const double wd);
  void		mrsim_motor_control(
    const struct mrsim_ids_simparam_s_motor_s *p,
    struct mrsim_ids_simstate_s_motor_s *motor);
  double	mrsim_motor_measure(
    const struct mrsim_ids_simstate_s_motor_s *motor);
  uint8_t	mrsim_seqid(struct timeval tv, double period);

#ifdef __cplusplus
}
#endif

#endif /* H_MRSIM_CODELS */
